<!DOCTYPE kpartgui>
<kpartgui name="rkward" version="0.4.2">
	<MenuBar>
		<Menu name="edit" noMerge="1"><text>&amp;Edit</text>
			<Menu name="tools"><text>&amp;Tools Move</text>
				<Merge/>
			</Menu>
			<Action name="copy"/>
			<Merge/>
		</Menu>
		<Menu name="run"><text>&amp;Run</text>
			<Action name="run_line"/>
			<Action name="run_selection"/>
			<Action name="run_all"/>
			<Separator/>
		</Menu>
		<Menu name="help"><text>&amp;Help</text>
			<Action name="function_reference"></Action>
		</Menu>
	</MenuBar>
	<ToolBar fullWidth="true" name="runToolBar">
		<Action name="run_line"/>
		<Action name="run_selection"/>
		<Action name="run_all"/>
	</ToolBar>
	<Menu name="ktexteditor_popup">
		<Merge/>
		<Separator/>
		<Action name="run_line"/>
		<Action name="run_selection"/>
		<Action name="run_all"/>
		<Separator/>
		<Action name="function_reference"/>
	</Menu>
</kpartgui>