#main {
	margin-right: 11em;
}

#main h1 {
	font-size: 1.5em;
	color: gray;
	text-align: center;
	margin-top: 1em;
	margin-bottom: 1em;
	padding: 0;
}

#main h2 {
	font-size: 1.2em;
	color: gray;
	margin-top: 1em;
	margin-bottom: 0.5em;
	padding: 0;
}

#main h3 {
	font-size: 1.1em;
	color: gray;
	margin-top: 1em;
	margin-bottom: 0em;
	padding: 0;
}

#main h4 {
	font-size: 1em;
	font-weight: bold;
	color: gray;
	margin-top: 0.5em;
	margin-bottom: 0em;
	padding: 0;
}

#main p {
	margin-top: 0em;
	margin-bottom: 0.5em;
	padding: 0;
}

#navigation {
	position: fixed;
	top: 1em;
	right: 0.5em;
	width: 10em;
	border: 1px solid silver;
}

#navigation p {
	font-size: 0.9em;
	margin-top: 0em;
	margin-bottom: 0.5em;
	text-indent: -1em;
	margin-left: 1.5em;
	margin-right: 0.2em;
}