<!DOCTYPE kpartgui>
<kpartgui name="rkeditordataframepart" version="0.3.3">
	<MenuBar>
		<Menu name="edit"><text>&amp;Edit</text>
			<Action name="cut"/>
			<Action name="copy"/>
			<Action name="paste"/>
			<Action name="paste_to_selection"/>
			<Action name="paste_to_table"/>
			<Separator/>
			<Merge/> 
		</Menu>
<!--		<Menu name="windows"><text>&amp;Windows</text>
			<Action name="window_close"/>
		</Menu>-->
	<ToolBar fullWidth="true" name="editToolBar">
		<Action name="cut"/>
		<Action name="copy"/>
		<Action name="paste"/>
		<Separator/>
		<Action name="paste_to_selection"/>
		<Action name="paste_to_table"/>
	</ToolBar>
</kpartgui>
