<!DOCTYPE kpartgui>
<kpartgui name="rkward" version="0.4.8">
	<Menubar>
		<Menu name="edit"><text>&amp;Edit</text>
			<Action name="log_select_all"/>
			<Action name="log_copy"/>
			<Action name="log_clear"/>
		</Menu>
		<Menu name="run"><text>&amp;Run</text>
			<Action name="log_run_selection"/>
		</Menu>
		<Menu name="settings"><text>&amp;Settings</text>
			<Action name="log_configure"/>
		</Menu>
	</Menubar>
	<Menu name="rkcommandlog_context_menu">
		<Action name="log_copy"/>
		<Action name="log_run_selection"/>
		<Separator/>
		<Action name="log_select_all"/>
		<Separator/>
		<Action name="log_clear"/>
		<Separator/>
		<Action name="log_configure"/>
	</Menu>
	<ToolBar fullWidth="true" name="runToolBar">
		<Action name="log_run_selection"/>
	</ToolBar>
</kpartgui>
