<!DOCTYPE kpartgui>
<kpartgui name="rkward" version="0.4.8">
	<MenuBar>
		<Merge/>
		<Menu name="window"><text>&amp;Window</text>
			<Merge/>
			<Action name="dwindow_close"/>
			<Separator/>
			<Menu name="window_activate"><text>&amp;Activate</text></Menu>
			<Separator/>
			<Action name="dwindow_attach"/>
		</Menu>
		<Menu name="help"><text>&amp;Help</text>
			<Merge/>
		</Menu>
	</MenuBar>
</kpartgui>
