<!DOCTYPE kpartgui>
<kpartgui name="rkward" version="0.4.8">
	<MenuBar>
		<Menu name="file"><text>&amp;File</text>
			<Action name="print_help"/>
		</Menu>
		<Menu name="edit"><text>&amp;Edit</text>
			<Action name="copy"/>
		</Menu>
		<Menu name="run"><text>&amp;Run</text>
			<Action name="run_selection"/>
		</Menu>
		<Merge/>
	</MenuBar>
	<ToolBar fullWidth="true" name="navigation_toolbar">
		<Action name="help_back"/>
		<Action name="help_forward"/>
	</ToolBar>
	<ToolBar name="runToolBar">
		<Action name="run_selection"/>
	</ToolBar>
</kpartgui>
