<!DOCTYPE kpartgui>
<kpartgui name="rkward" version="0.4.2">
	<MenuBar>
		<Menu name="device"><text>&amp;Device</text>
			<Action name="device_activate"/>
			<Action name="device_duplicate"/>
			<Separator/>
			<Action name="device_print"/>
			<Action name="device_copy_to_output"/>
			<Action name="device_copy_to_r_object"/>
			<Merge/>
		</Menu>
		<Menu name="view"><text>&amp;View</text>
			<Action name="toggle_fixed_size"/>
			<Separator/>
			<Action name="set_fixed_size_1"/>
			<Action name="set_fixed_size_2"/>
			<Action name="set_fixed_size_3"/>
			<Action name="set_fixed_size_manual"/>
			<Merge/>
		</Menu>
		<Merge/>
	</MenuBar>
</kpartgui>