<!DOCTYPE kpartgui>
<kpartgui name="rkward" version="0.4.8">
	<MenuBar>
		<Menu name="file"><text>&amp;File</text>
			<Action name="print_output"/>
		</Menu>
		<Menu name="edit"><text>&amp;Edit</text>
			<Action name="copy"/>
			<Separator/>
			<Action name="output_flush"/>
		</Menu>
		<Menu name="view"><text>&amp;View</text>
			<Separator/>
			<Action name="output_refresh"/>
		</Menu>
	</MenuBar>
	<ToolBar fullWidth="true" name="mainToolBar">
		<Action name="output_flush"/>
		<Action name="output_refresh"/>
	</ToolBar>
</kpartgui>
